/*
 * Jail Run - run program in chroot
 *
 * usage: <path>/jrun [ /setup | program args ]
 *
 * Copyright (c) 2007 Karl Brose
 *
 * This program is free software. It is available
 * under the GNU General Public License 2.0
 *
 * $Id: jrun.c,v 1.2 2007/11/05 05:30:48 nasmaker Exp $
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <limits.h>


int
main (int argc, char** argv)
{
	char	path[PATH_MAX];
	char*	name = argv[0];
	char*	s;
	char*	e;
	char*	d = path;

	if (*name != '/') {
		if (!getcwd(path, sizeof path)) {
			fprintf(stderr, "Error %d: Cannot get current working directory\n", errno);
			exit(errno);
		}
		while (*d) d++;
		*d++ = '/';
	}
	for (s=name, e=NULL; *s && d < path + PATH_MAX; d++, s++) {
		*d = *s;
		if (*d == '/') e = d;
	}
	if (e) *e = '\0';

	if (--argc && !strcmp(argv[1], "/setup")) {
		strcat(path, argv[1]);
		if (chdir(path)) {
			fprintf(stderr, "Error %d: %s not found.\n", errno, path);
			exit(errno);
		}
		strcat(path, argv[1]);
		execl(path, path);
		fprintf(stderr, "Error %d: Installation failed.\n", errno);
		exit(errno);
	}
	if (chroot(path)) {
		fprintf(stderr, "Error %d: Cannot change to root directory %s\n", errno, path);
		exit(errno);
	}
	if (chdir("/")) {
		fprintf(stderr, "Error: %d: Cannot set working directory\n", errno);
		exit(errno);
	}
	if (argc) {
		argv++;
		execvp(argv[0], argv);
	} else {
		char* argv[3] = { NULL, NULL, NULL };
		char* s = getenv("SHELL");
		argv[0] = s ? s : "/bin/sh";
		argv[1] = "-i";
		setenv("PS1", "JRUN\\w# ", 1);
		execvp(argv[0], argv);
	}
	fprintf(stderr, "Error %d: Execution failed.\n", errno);
	exit(errno);
}
=